package cz.cvut.agents.rph.reversi;

import cz.cvut.agents.rph.core.GameController;
import cz.cvut.agents.rph.core.GameModel;

import java.awt.event.ActionEvent;
import java.util.concurrent.*;

/**
 * Controller of the Reversi game.
 * Author: Ondrej Vanek
 * Date: 10/3/12
 * Time: 10:04 AM
 */
public class ReversiController extends GameController {

    /**
     * Delay in ms after each move.
     */
    protected int afterMoveDelay = 100;

    /**
     * Model of the game
     */
    protected ReversiModel reversiModel;

    /**
     * What is the timeout for one move?
     */
    private long timeoutInMilis;

    /**
     * First player.
     */
    protected ReversiPlayerInterface player1;

    /**
     * Second Player.
     */
    protected ReversiPlayerInterface player2;

    /**
     * Who is the first player.
     */
    protected ReversiPlayerInterface currentPlayer;

    static final String PLAY_PAUSE_ACTION_COMMAND = "playPause";
    static final String AFTER_MOVE_DELAY_CHANGED = "changeDelay";
    public static final String RESTART_ACTION_COMMAND = "restart";


    /**
     * Constructs the controller for the Reversi game.
     * @param model Reversi game model.
     * @param timeoutInMilis Timeout per move
     * @param player1 Player 1.
     * @param player2 Player 2.
     * @throws Exception Of course, something can go wrong - catch this guy to know what was it!
     */
    public ReversiController(ReversiModel model, long timeoutInMilis, ReversiPlayerInterface player1, ReversiPlayerInterface player2) throws Exception {
        super(model);
        this.reversiModel = model;
        this.timeoutInMilis = timeoutInMilis;
        this.player1 = player1;
        this.player2 = player2;
        model.setPlayerNames(player1.getName(),player2.getName());
    }

    /**
     * Something happened in the View - what was that?
     */
    public void actionPerformed(ActionEvent e) {
        if(e.getActionCommand().equals(PLAY_PAUSE_ACTION_COMMAND)){
            reversiModel.changePlayPauseGameStatus();
        } else if(e.getActionCommand().equals(AFTER_MOVE_DELAY_CHANGED)){
            afterMoveDelay = ((DelayChanger)e.getSource()).getDelayInMiliseconds();

        }
    }

    /**
     * DUH, restart the game.
     */
    public void restartGame() throws Exception{
        reversiModel.restart();
        switchPlayers();
        reversiModel.setPlayerNames(player1.getName(),player2.getName());
        currentPlayer = player1;
    }
    @Override
    public ReversiGameResult run() throws Exception {
        return run(false);
    }


    /**
     * Runs the game
     * @param autostart set to true if you want the game to start immediately - typically, when no view is used.
     * @return result of the game.
     * @throws Exception - if something goes wrong, catch it!
     */
    public ReversiGameResult run(boolean autostart) throws Exception {
        if(autostart){
            reversiModel.setGameStatus(GameModel.GameStatus.PLAYING);
        }
        currentPlayer = player1;

        ReversiGameResult result;  //hold the result.
        while(true){   //game loop
            if(reversiModel.getGameStatus().equals(GameModel.GameStatus.PLAYING)){ //if we should play, let's play
                if(reversiModel.isEndOfGame()){ // check end of game
                    result = endGame(null, null);   //everything should be OK - no error or timeout.
                    break;
                }
                if(reversiModel.canPlay(currentPlayer.getMyColor())){ //if the current player can play, let him make a move.
                    ReversiMove move;
                    try {     //try to make move
                        move = makeNextMove(currentPlayer);
                    } catch (TimeoutException e) { // if there is timeout, catch and end game!
                        System.err.println("Timeout by "+ currentPlayer.getName());
                        result = endGame(currentPlayer, null);
                        break;
                    }  catch (ExecutionException e){ //catching Exceptions thrown by the palyer (or Heap errors wrapped by this Exception guy).
                        System.err.println("Exception/Error thrown by "+ currentPlayer.getName() + ":");
                        e.getCause().printStackTrace(System.err);
                        result = endGame(null, currentPlayer);
                        break;
                    }
                    if(reversiModel.isCorrectMove(currentPlayer.getMyColor(),move)){ //if correct, place it and switch players
                        reversiModel.putMove(currentPlayer.getMyColor(),move);
                        afterMoveDelay();
                        reversiModel.confirmMoves();
                        changeCurrentPlayer();
                    }else{          // incorrect move, end game.
                        System.err.println("Wrong move by "+currentPlayer.getName()+" :"+move);
                        result = endGame(null, currentPlayer);
                        break;
                    }

                }else{
                    changeCurrentPlayer();
                    if(!reversiModel.canPlay(currentPlayer.getMyColor())){
                        result = endGame(null,null);
                        break;
                    }
                }
            }
            Thread.sleep(1);   // this sleep is for the eventQueue to get priority (view updates are Swing.invokeLater)
        }
        return result;
    }


    /**
     * Method wrapping the endgame procedure - creation of result, update of model, etc.
     * @param timeoutPlayer who was the timeout player? Set to <code>null</code> if there wasn't one.
     * @param errorPlayer who was the error player? Set to <code>null</code> if there wasn't one.
     * @return result of the game.
     * @throws Exception well, you know...
     */
    public ReversiGameResult endGame(ReversiPlayerInterface timeoutPlayer, ReversiPlayerInterface errorPlayer) throws Exception {
        boolean player1Timeout = false;
        boolean player2timeout = false;
        boolean player1Error = false;
        boolean player2Error = false;

        String timeoutPlayerName="";
        String errorPlayerName="";

        if(timeoutPlayer!=null){
            if(timeoutPlayer==player1){
                player1Timeout = true;
                timeoutPlayerName = player1.getName();
            }else{
                player2timeout = true;
                timeoutPlayerName = player2.getName();
            }
        }
        if(errorPlayer!=null){
            if(errorPlayer==player1){
                player1Error = true;
                errorPlayerName = player1.getName();
            }else{
                player2Error = true;
                errorPlayerName = player2.getName();
            }
        }
        ReversiGameResult gameResult = ReversiScoreEvaluator.evaluate(
                reversiModel.getPlayer1Stones(),reversiModel.getPlayer2Stones(),
                player1Timeout,player2timeout,
                player1Error,player2Error,
                player1.getName(),player2.getName());

        reversiModel.setGameResult(gameResult,errorPlayerName,timeoutPlayerName); //update the model (show in the GUI
        reversiModel.setGameStatus(GameModel.GameStatus.FINISHED); //this will notify the gui as well, displays the dialog
        return gameResult;

    }

    /**
     * PURE MAGIC! :)
     */
    protected void changeCurrentPlayer() {
        if(currentPlayer.equals(player1)){
            currentPlayer = player2;
        }else{
            currentPlayer = player1;
        }
    }

    /**
     * Switches first and second player. (typically after game restart).
     */
    protected void switchPlayers() throws Exception{
        ReversiPlayerInterface tmp = player1;
        player1 = player2;
        player2 = tmp;

        player1.systemInit(reversiModel.getWidth(),reversiModel.getHeight(),ReversiModel.PLAYER1_COLOR,ReversiModel.PLAYER2_COLOR);
        player2.systemInit(reversiModel.getWidth(),reversiModel.getHeight(),ReversiModel.PLAYER2_COLOR,ReversiModel.PLAYER1_COLOR);
    }

    /**
     * Wrapping of next move calling. Using {@link FutureTask} to get the move after given timeout or sooner
     * @param currentPlayer   who is playing.
     * @return Move of the currentPlayer.
     * @throws TimeoutException thrown if the player didn't decide in time
     * @throws ExecutionException thrown if the was an Exception/Error thrown by the player.
     * @throws InterruptedException thrown when the FutureTask has been interrupted. Shouldn't been happening.
     */
    protected ReversiMove makeNextMove(final ReversiPlayerInterface currentPlayer) throws TimeoutException, ExecutionException, InterruptedException {

        //Creation of task
        FutureTask<ReversiMove> task = new FutureTask<ReversiMove>(new Callable<ReversiMove>() {
            public ReversiMove call() throws Exception {
                return currentPlayer.makeNextMove(reversiModel.getBoardCopy());
            }
        });

        ExecutorService executor = Executors.newSingleThreadExecutor();

        // Submiting the task and waiting for result.
        long time = System.currentTimeMillis();
        executor.submit(task);
        ReversiMove reversiMove = task.get(timeoutInMilis, TimeUnit.MILLISECONDS);
        long time2 = System.currentTimeMillis();

        if(time2-time > timeoutInMilis){
            System.out.println("Time taken to make move: "+ (time2-time));
        }
        if(reversiMove==null){ // it is handled later
            return reversiMove;
        }
        reversiMove.setTimeInMilis(time2-time); //sets the time duration of the move.
        return reversiMove;
    }

    /**
     * We should sleep a bit so that the user actually sees something from the game.
     * @throws InterruptedException
     */
    protected void afterMoveDelay() throws InterruptedException {
        try {
            Thread.sleep(afterMoveDelay);
        } catch (InterruptedException e) {
            e.printStackTrace();
            throw  e;
        }
    }

    /**
     * sets the delay between moves to defined value in miliseconds.
     * @param afterMoveDelay
     */
    public void setAfterMoveDelay(int afterMoveDelay) {
        this.afterMoveDelay = afterMoveDelay;
    }
}
