package cz.cvut.agents.rph.reversi;

import cz.cvut.agents.rph.core.GameModel;

/**
 * Concrete instance of the Reversi game model.
 * Author: Ondrej Vanek
 * Date: 10/3/12
 * Time: 10:05 AM
 */
public class ReversiModel extends GameModel {

    protected int[][] board;
    protected int width;
    protected int height;

//    protected ReversiPlayer player1;
//    protected ReversiPlayer player2;

    protected long player1MaxTime = 0;
    protected long player2MaxTime = 0;

    private String player1Name = "";
    private String player2Name = "";

    ReversiGameResult gameResult=null;
    private String errorPlayerName="";
    private String timeoutPlayerName="";

    public static final int EMPTY_SQUARE=-1;
    public static final int PLAYER1_COLOR=0;
    public static final int PLAYER2_COLOR=1;


    @Override
    public boolean isEndOfGame() {
        if(getStonesWithColor(PLAYER1_COLOR)+getStonesWithColor(PLAYER2_COLOR)==width*height){
            return true;
        }
        if(!canPlay(PLAYER1_COLOR) && !canPlay(PLAYER2_COLOR)){
            return true;
        }
        return false;

    }

    public void setPlayerNames(String player1Name, String player2Name){
        this.player1Name = player1Name;
        this.player2Name = player2Name;
    }

    public String getPlayer1Name() {
        return player1Name;
    }

    public String getPlayer2Name() {
        return player2Name;
    }

    public boolean canPlay(int player) {
        for (int i=0;i<width;i++) {
            for (int j=0;j<height;j++) {
                ReversiMove move = new ReversiMove(i, j);
                if ((board[i][j] == EMPTY_SQUARE) && (isCorrectMove(player, move))) {
                    return true;
                }
            }
        }
        return false;
    }

    public void changePlayPauseGameStatus() {
        if(gameStatus.equals(GameStatus.PAUSED)){
            setGameStatus(GameStatus.PLAYING);
        }
        else if(gameStatus.equals(GameStatus.PLAYING)){
            setGameStatus(GameStatus.PAUSED);
        }
        notifyViews();

    }

    /**
     * Sets the game result AND notify the views to react.
     * @param gameResult
     * @param errorPlayerName
     * @param timeoutPlayerName
     */
    public void setGameResult(ReversiGameResult gameResult,String errorPlayerName, String timeoutPlayerName) {
        this.gameResult = gameResult;
        this.errorPlayerName = errorPlayerName;
        this.timeoutPlayerName = timeoutPlayerName;
        notifyViews();
    }

    @Override
    public void restart() {
        initPlayground();
        gameResult=null;
        setGameStatus(GameStatus.PAUSED);
        player1MaxTime = 0;
        player2MaxTime = 0;
        player1Name = "";
        player2Name = "";
        errorPlayerName="";
        timeoutPlayerName="";
        notifyViews();

    } ;

    public int[][] getBoardCopy(){
         return copyBoard(board);
    }

    protected int[][] copyBoard(int[][] board){
        int[][] copy = new int[board.length][board[0].length];

        for(int i =0;i<board.length;i++){
            for(int j=0;j<board[0].length;j++){
                copy[i][j] = board[i][j];
            }
        }
        return copy;

    }

    int[][] getBoard() {
        return board;
    }

    public ReversiModel(int width, int height){
        this.width = width;
        this.height = height;
        initPlayground();
    }

   int getPlayer1Stones(){
        return getStonesWithColor(PLAYER1_COLOR);
    }

    int getPlayer2Stones(){
        return getStonesWithColor(PLAYER2_COLOR);
    }

    private int getStonesWithColor(int color) {
        int sum=0;
        for (int x=0; x< width; x++) {
            for (int y=0; y< width; y++) {
                if(board[x][y]==color){
                    sum++;
                }
            }
        }
        return sum;
    }


    public int getWidth() {
        return width;
    }


    public int getHeight() {
        return height;
    }


    private void initPlayground() {
        board = new int[width][height];
        for (int x=0; x< width; x++) {
            for (int y=0; y< width; y++) {
                board[x][y]= EMPTY_SQUARE;
            }
        }
        board[width /2-1][height /2-1]=PLAYER1_COLOR;
        board[width /2][height /2]=PLAYER1_COLOR;
        board[width /2][height /2-1]=PLAYER2_COLOR;
        board[width /2-1][height /2]=PLAYER2_COLOR;
    }

    public int getValue(int x, int y) {
        return board[x][y];
    }

    @Override
    public ReversiGameResult getGameResult() {
        return gameResult;
    }

    /**
     * Puts the move of a player to the board. Places temporary color onto the board. Call confirmMoves to set the
     * board values to the normal 0,1 values.
     * @param player
     * @param move
     */
    public void putMove(int player, ReversiMove move){
        int x= move.getX();
        int y=  move.getY();

        changeStonesInDirection(player, 1, 0, x, y);
        changeStonesInDirection(player, 1, 1, x, y);
        changeStonesInDirection(player, 0, 1, x, y);
        changeStonesInDirection(player, -1, 1, x, y);
        changeStonesInDirection(player, -1, 0, x, y);
        changeStonesInDirection(player, -1, -1, x, y);
        changeStonesInDirection(player, 0, -1, x, y);
        changeStonesInDirection(player, 1, -1, x, y);

        board[x][y] = player + 2;

        updateMaxTimes(player,move);

        notifyViews();
    }

    private void updateMaxTimes(int player, ReversiMove move) {
        if(player==PLAYER1_COLOR){
            if(player1MaxTime<move.getTimeInMilis()){
                player1MaxTime=move.getTimeInMilis();
            }
        }else{
            if(player2MaxTime<move.getTimeInMilis()){
                player2MaxTime=move.getTimeInMilis();
            }
        }
    }

    public long getPlayer1MaxTime() {
        return player1MaxTime;
    }

    public long getPlayer2MaxTime() {
        return player2MaxTime;
    }

    private void changeStonesInDirection(int player, int dirX, int dirY, int x, int y) {
        boolean neighbourOpponent = false;
        boolean endingMyStone = false;
        int steps = 0;
        while (!endingMyStone) {
            x += dirX;
            y += dirY;
            steps++;
            if (x<0 || x>=width || y<0 || y>=height) {
                break;
            }
            int color = board[x][y];
            if (color<0) {
                break;
            }
            if (color!=player) {
                neighbourOpponent = true;
            }
            else {
                endingMyStone = true;
            }
        }
        if (neighbourOpponent && endingMyStone) {
            dirX *= -1;
            dirY *= -1;
            for (int i=1;i<steps;i++) {
                x += dirX;
                y += dirY;
                board[x][y] = player + 4;
            }
        }
        notifyViews();
    }

    /**
     * Puts -1,0,1 to the board, instead of 2,3,4,5 (temporary colors).
     */
    public void confirmMoves() {
        for (int x=0; x<width; x++) {
            for (int y=0; y<height; y++) {
                int color = board[x][y];
                if (color<=1) {
                    continue;
                }
                int newColor = color % 2;
                board[x][y]=newColor;
            }
        }
        notifyViews();
    }

    public boolean isCorrectMove(int player, ReversiMove move) {
        if(move==null) return  false;
        int x = move.getX();
        int y = move.getY();

        if (x<0 || x>=width || y<0 || y>=height) {
            return false;
        }
        if (board[x][y]>=0) {
            return false;
        }
        boolean direction = false;
        direction |= isCorrectMoveInDirection(player, 1, 0, x, y);
        direction |= isCorrectMoveInDirection(player, 1, 1, x, y);
        direction |= isCorrectMoveInDirection(player, 0, 1, x, y);
        direction |= isCorrectMoveInDirection(player, -1, 1, x, y);
        direction |= isCorrectMoveInDirection(player, -1, 0, x, y);
        direction |= isCorrectMoveInDirection(player, -1, -1, x, y);
        direction |= isCorrectMoveInDirection(player, 0, -1, x, y);
        direction |= isCorrectMoveInDirection(player, 1, -1, x, y);

        return direction;
    }

    private boolean isCorrectMoveInDirection(int player, int dirX, int dirY, int x, int y) {
        boolean neighbourOpponent = false;
        boolean endingMyStone = false;
        while (!endingMyStone) {
            x += dirX;
            y += dirY;
            if (x<0 || x>=width || y<0 || y>=height) {
                return false;
            }
            int color = board[x][y];
            if (color==EMPTY_SQUARE) {
                return false;
            }
            if (color!=player) {
                neighbourOpponent = true;
            }
            else {
                endingMyStone = true;
            }
        }
        if (neighbourOpponent && endingMyStone) {
            return true;
        }
        return false;
    }

    public String getTimeoutPlayerName() {
        return timeoutPlayerName;
    }

    public String getErrorPlayerName() {
        return errorPlayerName;
    }
}
