package cz.cvut.agents.rph.reversi;

import java.rmi.Remote;
import java.rmi.RemoteException;

/**
 * Interface for a reversi player. Extends {@link Remote} interface to be able to play remotely.
 * Author: Ondrej Vanek
 * Date: 10/4/12
 * Time: 1:11 PM
 */
public interface ReversiPlayerInterface extends Remote {

    /**
     * Initializes the player with game parameters. You can override this method to do your own initialization.
     * @param boardWidth WIDTH of the board
     * @param boardHeight HEIGHT of the board
     * @param myColor color of the player
     * @param opponentColor color of the opponent
     */
    public void init(int boardWidth, int boardHeight, int myColor, int opponentColor) throws RemoteException;


    /**
     * Initialization method called by the game system.
     * @param boardWidth
     * @param boardHeight
     * @param myColor
     * @param opponentColor
     * @throws RemoteException
     */
    public void systemInit(int boardWidth, int boardHeight, int myColor, int opponentColor) throws RemoteException;



    /**
     * Gets the color of the player.
     * @return color of the player.
     */
    public int getMyColor() throws RemoteException;

    /**
     * Gets the color of the opponent player.
     * @return  color of the opponent player.
     */
    public int getOpponentColor() throws RemoteException;

    /**
     * Gets the color of the empty square.
     * @return  color of the empty square.
     */
    public int getEmptySquareColor() throws RemoteException;

    /**
     * Returns name of the reversi player. Return your username!
     * @return name of the reversi player
     */
    public String getName() throws RemoteException;

    /**
     * makeNextTurn is called by the reversi server when the player has to compute the next move.
     * The method returns a {@link ReversiMove} class encapsulating the coordinates of the stone to be put on the board.
     * @param board - contains actual map of the board represented by two-dimensional matrix.
     *              Each sqaure contains number of -1 representing empty square and number 0 and 1 representing players' stones.
     * @return specific reversiMove
     */
    public ReversiMove makeNextMove(int[][] board) throws RemoteException;

    /**
     * Signalizes end of the game.
     */
    public void endGame() throws RemoteException;

}
