package cz.cvut.agents.rph.reversi.main;

import cz.cvut.agents.rph.reversi.ReversiController;
import cz.cvut.agents.rph.reversi.ReversiGameResult;
import cz.cvut.agents.rph.reversi.ReversiModel;
import cz.cvut.agents.rph.reversi.ReversiPlayer;
import cz.cvut.agents.rph.reversi.players.DummyPlayer;

import java.util.LinkedList;
import java.util.List;

/**
 * Creator of the Reversi game without the GUI {@link cz.cvut.agents.rph.reversi.ReversiView}. No restarts.
 * Author: Ondrej Vanek
 * Date: 10/3/12
 * Time: 10:09 AM
 */
public class HeadlessReversiCreator {

    /**
     * Width of the playground.
     */
    protected static final int DEFAULT_PLAYGROUND_WIDTH = 8;

    /**
     * Height of the playground.
     */
    protected static final int DEFAULT_PLAYGROUND_HEIGHT = 8;

    /**
     * timeout for one move in miliseconds.
     */
    protected static int DEFAULT_TIMEOUT_MS = 2000;

    /**
     * Model of the game.
     */
    protected ReversiModel model;

    /**
     * Controller of the game.
     */
    protected ReversiController controller;

    /**
     * Creates a Reversi game. Players can be specified in the arguments
     * @param args full names of the player classes (e.g. cz.cvut.agents.rph.reversi.players.WrongMovePlayer)
     * and the optional arguments --timeout=, --width= and --height=.
     * @throws Exception
     */
    public static void main(String[] args) throws Exception {
        HeadlessReversiCreator creator = new HeadlessReversiCreator();

        Options options = processOptionalArguments(args);
        args = options.remainingArgs;

        ReversiPlayer player1 = null;
        ReversiPlayer player2 = null;

        player1 = new DummyPlayer();
        player2 = new DummyPlayer();

        if(args.length>0){
            try {
                player1 = (ReversiPlayer) Class.forName(args[0]).newInstance();
                System.out.println("Instantiated player1: " + args[0]);
            } catch (Exception e) {
                System.err.println("Cannot instantiate class: "+ args[0]+". Didn't you forget packages?");
                System.exit(99);
            }
        }

        if(args.length>1){
            try {
                player2 = (ReversiPlayer) Class.forName(args[1]).newInstance();
                System.out.println("Instantiated player2: " + args[1]);
            } catch (Exception e) {
                System.err.println("Cannot instantiate class: "+ args[1]+". Didn't you forget packages?");
                System.exit(99);
            }
        }

        creator.createGame(options.width, options.height, player1, player2, options.timeoutInMilis);

        try {
            ReversiGameResult gameResult = creator.playGame();
            gameResult.show();
            System.exit(gameResult.proceededWithoutErrorsAndTimeouts() ? 33 : 1);

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    protected static class Options {
        int timeoutInMilis;
        int width;
        int height;
        String[] remainingArgs;
    }

    protected static Options processOptionalArguments(String[] args) {
        List<String> passthroughArgs = new LinkedList<String>();

        Options options = new Options();
        options.timeoutInMilis = DEFAULT_TIMEOUT_MS;
        options.width = DEFAULT_PLAYGROUND_WIDTH;
        options.height = DEFAULT_PLAYGROUND_HEIGHT;

        for (String arg : args) {
            if (arg.startsWith("--timeout=")) {
                options.timeoutInMilis = Integer.parseInt(arg.substring("--timeout=".length()));
            } else  if (arg.startsWith("--width=")) {
                options.width = Integer.parseInt(arg.substring("--width=".length()));
            } else  if (arg.startsWith("--height=")) {
                options.height = Integer.parseInt(arg.substring("--height=".length()));
            } else {
                passthroughArgs.add(arg);
            }
        }

        options.remainingArgs = passthroughArgs.toArray(new String[0]);

        return options;
    }

    protected ReversiController createGame(int width, int height, ReversiPlayer player1, ReversiPlayer player2, int timeoutInMilis) throws Exception {
        player1.systemInit(width,height, ReversiModel.PLAYER1_COLOR, ReversiModel.PLAYER2_COLOR);
        player2.systemInit(width, height, ReversiModel.PLAYER2_COLOR, ReversiModel.PLAYER1_COLOR);

        //Initialize reversi game model, controller,view
        model = new ReversiModel(width,height);
        controller = new ReversiController(model, timeoutInMilis, player1, player2);

        return controller;
    }

    /**
     * Plays the game with autorun enabled.
     * @throws Exception
     */
    private ReversiGameResult playGame() throws Exception {
        controller.setAfterMoveDelay(1);
        ReversiGameResult result = controller.run(true);
        return result;
    }
}
