package cz.cvut.agents.rph.reversi.main;

import cz.cvut.agents.rph.reversi.*;
import cz.cvut.agents.rph.reversi.players.DummyPlayer;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * Creator of the Reversi game. Main entry point of the game.
 * Author: Ondrej Vanek
 * Date: 10/3/12
 * Time: 10:09 AM
 */
public class ReversiCreator extends HeadlessReversiCreator implements ActionListener {

    /**
     * Width of the JFrame holding the {@link cz.cvut.agents.rph.reversi.ReversiView}.
     */
    protected static final int FRAME_WIDTH = 850;

    /**
     * Height of the JFrame holding the {@link cz.cvut.agents.rph.reversi.ReversiView}.
     */
    protected static final int FRAME_HEIGHT = 400;

    /**
     * View of the game
     */
    protected ReversiView view;


    /**
     * Creates a Reversi game. Players can be specified in the arguments
     * @param args full names of the player classes (e.g. cz.cvut.agents.rph.reversi.players.WrongMovePlayer)
     * and the optional arguments --timeout=, --width= and --height=.
     * @throws Exception
     */
    public static void main(String[] args) throws Exception {

        ReversiCreator creator = new ReversiCreator();

        Options options = processOptionalArguments(args);
        args = options.remainingArgs;

        ReversiPlayer player1 = null;
        ReversiPlayer player2 = null;

        player1 = new DummyPlayer();
        player2 = new DummyPlayer();

        if(args.length>0){
            try {
                player1 = (ReversiPlayer) Class.forName(args[0]).newInstance();
                System.out.println("Instantiated player1: " + args[0]);
            } catch (Exception e) {
                System.err.println("Cannot instantiate class: "+ args[0]+". Didn't you forget packages?");
                System.exit(99);
            }
        }

        if(args.length>1){
            try {
                player2 = (ReversiPlayer) Class.forName(args[1]).newInstance();
                System.out.println("Instantiated player2: " + args[1]);
            } catch (Exception e) {
                System.err.println("Cannot instantiate class: "+ args[1]+". Didn't you forget packages?");
                System.exit(99);
            }
        }

        creator.createGame(options.width, options.height, player1, player2, options.timeoutInMilis);

        try {
            creator.playGame();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Creates the whole game.
     * @param width width of the playground.
     * @param height height of the playground.
     * @param player1 first player.
     * @param player2 second player.
     * @return controller responsible for the game execution - for restarting purposes, accessible as field as well.
     * @throws Exception if something goes wrong, inspect this guy.
     */
    protected ReversiController createGame(int width, int height, ReversiPlayer player1, ReversiPlayer player2, int timeoutInMilis) throws Exception {
        setLookAndFeel();

        player1.systemInit(width,height, ReversiModel.PLAYER1_COLOR, ReversiModel.PLAYER2_COLOR);
        player2.systemInit(width, height, ReversiModel.PLAYER2_COLOR, ReversiModel.PLAYER1_COLOR);

        //Initialize reversi game model, controller,view
        model = new ReversiModel(width,height);
        controller = new ReversiController(model, timeoutInMilis,player1,player2);
        view = new ReversiView(model);

        // attach view to the model.
        model.addView(view);
        //listen to the actions coming from the view (i.e. the user).
        view.addActionListener(controller);
        view.addActionListener(this);

        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                JFrame frame = new JFrame("Reversi");
                frame.getContentPane().add(view.getPanel());
                frame.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);

                frame.pack();
                frame.setSize(FRAME_WIDTH,FRAME_HEIGHT);
                frame.setVisible(true);
            }
        });

        return controller;
    }

    /**
     * Calls the controller with to execute the game.
     * @return result of the game.
     * @throws Exception
     */
    private ReversiGameResult playGame() throws Exception {
        if(controller==null){
            throw new RuntimeException("Controller not initialized!");
        }

        ReversiGameResult result = controller.run();
        result.show();
        return result;
    }


    /**
     * For restarting purposes.
     * @param e event (from the {@link ReversiView}.
     */
    public void actionPerformed(ActionEvent e) {
        if(e.getActionCommand().equals(ReversiController.RESTART_ACTION_COMMAND)){
            ExecutorService executorService = Executors.newFixedThreadPool(1);
            executorService.execute(new Runnable() {
                public void run() {
                    try {
                        controller.restartGame();
                        playGame();
                    } catch (Exception e1) {
                        throw new RuntimeException(e1);
                    }
                }
            });
        }
    }


    /**
     * Sets nicer look&feel (Nimbus - works from Java 6.10 further).
     */
    private void setLookAndFeel() {
        try {
            SwingUtilities.invokeAndWait(new Runnable() {
                public void run() {
                    try {
                        UIManager.LookAndFeelInfo plafinfo[] = UIManager.getInstalledLookAndFeels();
                        boolean nimbusFound=false;
                        int nimbusindex=0;

                        for (int look = 0; look < plafinfo.length; look++){
                            if(plafinfo[look].getClassName().toLowerCase().contains("nimbus")){
                                nimbusFound=true;
                                nimbusindex=look;
                            }
                        }
                        if (nimbusFound)	{
                            UIManager.setLookAndFeel(plafinfo[nimbusindex].getClassName());
                        } else {
                            System.out.println("Nimbus look&feel not found. Using System Look&Feel.");
                            UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
                        }
                    } catch (Exception ex) {
                        System.out.println("Better look&feel not found. Using the ugly one.");
                    }
                }

            });
        } catch (Exception e) {
            System.err.println("Error when setting look and feel - thread interrupted.");
            //e.printStackTrace();
        }

    }
}
